# R code for the modelling procedure
# Author: Edgar J. González
# Date of creation: 30/08/2016

# Loading the required packages
library(gdata)
library(gtools) 
library(sampling)

setwd("...") # set working directory
model.data = read.xls("Block et al Supplementary Material 3.xls", 
 sheet = 1, header = TRUE) # read first sheet of the .xls datafile

image = "G" # either "G" (Google) or "S" (SPOT), depending on the image data we want to use
nx = length(which(substr(names(model.data),1,1) == image)) # number of explanatory variables derived from Google Earth imagery (those starting with a G in the name of the column in model.data, we're assuming that the number of explanatory variables is the same for Google and SPOT)
ny = length(which(substr(names(model.data),1,1) == "Y")) # number of dependent variables (those starting with a Y in the name of the column in model.data)
combs.data = combn(n,d) # all possible combinations of data set to use in the cross-validation process

model.data[,which(substr(names(model.data),1,2) == "YP")] = 
 log(model.data[,which(substr(names(model.data),1,2) == "YP")]) # log-transformation of positive variables (those starting with a YP in the name of the column in model.data)
model.data[,which(substr(names(model.data),1,2) == "YN")] =
 log(model.data[,which(substr(names(model.data),1,2) == "YN")]+1) # log-transformation of non-negative variables (those starting with a YN in the name of the column in model.data)

## MODEL FITTING PROCEDURE ##

formula = y ~ x1 # formula of the model we want to fit
n.par = 2 # number of parameters in the model
n.x = 1 # number of explanatory variables in the model (either 1, 2 or 3)
combs = combn(nx,n.x) # all possible combinations of explanatory variables
n.combs = choose(nx,n.x)

results = list() # set the list with results
length(results) = ny*n.combs
for (i in 1:ny) { # for each dependent variable...
	y = model.data[, which(substr(names(model.data),1,1) == "Y")[i]]
	for(j in 1:n.combs) { # ... and each possible combination of explanatory variables...
  x1 = model.data[, which(substr(names(model.data),1,1) == image)[combs[1,j]]]
  if (n.x > 1) 
   x2 = model.data[, which(substr(names(model.data),1,1) == image)[combs[2,j]]]
  if (n.x > 2)
   x3 = model.data[, which(substr(names(model.data),1,1) == image)[combs[3,j]]]
  results[[n.combs*(i-1)+j]] = lm(formula) # ... fit the model with the given formula and store it into the list of results, ...
 }
}
file.name = paste0("Models_",image,"x",n.x,"p",n.par)
save(results, file = paste0(file.name, ".Rdata")) # ... and store the list in an .Rdata file

## CROSS-VALIDATION (CV) PROCEDURE ##
 
# Function to get the i-th possible combination of taking r elements from n available without replacement.
# Code for this function taken from: http://msdn.microsoft.com/en-us/library/aa289166(VS.71).aspx
combinadic = function(n,r,i) {
	if(i < 1 | i > choose(n,r))
  stop("'i' must be 0 < i <= n!/(n-r)!")
 largestV = function(n, r, i) {
  v = n
  while(choose(v,r) >= i)
  	v = v-1
  return(v)
	}
 res = rep(NA,r)
 for(j in 1:r) {
  res[j] = largestV(n,r,i)
  i = i-choose(res[j],r)
  n = res[j]
  r = r-1
 }
 res = res + 1
 return(res)
}

n = dim(model.data)[1] # number of observed data
d = 2 # number of data to leave out in the cross-validation process (by setting d = 2 we're performing a leave-two-out CV)        
combs.max = choose(n,d) # total number of randomized samples of size n-d of the data. Note that if this number is smaller than choose(n,d) this section of the code is no longer reproducible
combs.data.CV = matrix(nrow = d,ncol = combs.max) # matrix of combinations of rows that would have to left out from the validation process
rands <- sample(seq(1,choose(n,d),1), combs.max) # get a (randomized) sample with no replacement of the possible randomized samples of size n-d of the data
for(i in 1:combs.max) # get the combs.max randomized samples
 combs.data.CV[,i] = combinadic(n,d,rands[i]) 

pred = vector(length = d) # vector of predicted values
r2.CV = vector(length = choose(n,d)) # cross-validation r2 for the combs.max randomized n-d samples of the data
means = matrix(data = 0,nrow = ny,ncol = choose(nx,n.x)) # matrix of r2.CV means for each dependent and possible combination of explanatory variables
        
for(i in 1:ny) { # for each dependent variable, ...
 y = model.data[, which(substr(names(model.data),1,1) == "Y")[i]]
 mean.y = mean(y)
	mean.sstt = sum((y-mean.y)^2)/n
 for(j in 1:n.combs) { # ... each possible combination of explanatory variables...
  for(k in 1:combs.max) { # ... and each randomized sample of size n-d of the data, ...
   validation.data = model.data[combs.data.CV[,k],] # ... divide model.data into a validation dataset (of size d)...
   calibration.data = model.data[-combs.data.CV[,k],]  # ... and a calibration dataset (of size n-d), ...
   y = calibration.data[, which(substr(names(model.data),1,1) == "Y")[i]]
   x1 = calibration.data[, which(substr(names(model.data),1,1) == image)[combs[1,j]]]
   if (n.x > 1) 
    x2 = calibration.data[, which(substr(names(model.data),1,1) == image)[combs[2,j]]]
   if (n.x > 2)
    x3 = calibration.data[, which(substr(names(model.data),1,1) == image)[combs[3,j]]]
   model = lm(formula) # ... fit the model to the calibration data, ...
   x1 = validation.data[, which(substr(names(model.data),1,1) == image)[combs[1,j]]]
   if (n.x > 1) 
    x2 = validation.data[, which(substr(names(model.data),1,1) == image)[combs[2,j]]]
   if (n.x > 2)
    x3 = validation.data[, which(substr(names(model.data),1,1) == image)[combs[3,j]]]
   if (n.x == 1)
    pred = predict(model, newdata = list(x1 = x1))
   if (n.x == 2)
    pred = predict(model, newdata = list(x1 = x1, x2 = x2))
   if (n.x == 3)
    pred = predict(model, newdata = list(x1 = x1, x2 = x2, x3 = x3)) # ... predict the dependent variable with the validation data, ...
   y = validation.data[, which(substr(names(model.data),1,1) == "Y")[i]]
   msse.validation = sum((y-pred)^2)/d
   r2.CV[k] = 1-msse.validation/mean.sstt # ... get the R2 of the observed vs predicted values of this dependent variable, ...
		}
  means[i,j] = mean(r2.CV) # ... save the mean of the R2.CV's for all the randomized samples, ...
	}
}
file.name = paste0("R2cv_",image,"x",n.x,"p",n.par)
save(means, file = paste0(file.name, ".Rdata")) # ... and store the matrix of means in an .Rdata file

## ESTIMATING THE P-VALUES OF THE OBTAINED R2S RELATIVE TO THE THEIR RANDOM DISTRIBUTIONS ##

runs = 1000 # number of permutations to be performed      
model.data.permut = model.data 
r2.max.permut = matrix(nrow = ny,ncol = runs) 
r2.model.pvalue = matrix(nrow = ny,ncol = choose(nx,1))
r2 = array(dim = c(ny,n.combs,runs))
for(k in 1:runs) { # on each run, ...
 for(i in 1:nx) { # ... for each explanatory variable ...
  l = which(substr(names(model.data),1,1) == image)[i]
  model.data.permut[,l] = permute(model.data[,l]) # ... permute its values; ...
 }
 for(i in 1:ny) { # ... then, for each dependent variable ...
  y = model.data.permut[, which(substr(names(model.data),1,1) == "Y")[i]]
  for(j in 1:n.combs) { # ... and each possible combination of explanatory variables, ...
   x1 = model.data.permut[, which(substr(names(model.data),1,1) == image)[combs[1,j]]]
   if (n.x > 1) 
    x2 = model.data.permut[, which(substr(names(model.data),1,1) == image)[combs[2,j]]]
   if (n.x > 2)
    x3 = model.data.permut[, which(substr(names(model.data),1,1) == image)[combs[3,j]]]
   model = lm(formula) # ... fit the model... 
   r2[i,j,k] = summary(model)$r.squared # ... and get its associated R2 value.
  }
 }
}
# In this loops we only determine the location of the R2 obtained from the original model (obtained in the Model Fitting Procedure) in the random R2 distribution and assign it a p-value
for(i in 1:ny) { 
	for(j in 1:n.combs) {
  r2[i,j,] = sort(r2[i,j,],decreasing = TRUE)
  r2 <- summary(results[[n.combs*(i-1)+j]])$r.squared
		if(r2 >= r2[i,j,1]) {
			r2.model.pvalue[i,j] = 0.0005 # i.e. r2.model.pvalue < 0.001
		}	else {
			if(r2 < r2[i,j,runs])	{
				r2.model.pvalue[i,j] = 1
		 } else {
    for(k in 2:runs) {
     if(r2 >= r2[i,j,k] && 
      r2 < r2[i,j,k-1]) {
      r2.model.pvalue[i,j] = (k-1)/runs
						break
					}
				}
			}
  }
	}
}
file.name = paste0("Pvalues_",image,"x",n.x,"p",n.par) 
save(r2.model.pvalue, file = paste0(file.name, ".Rdata")) # We store the matrix of p-values in an .Rdata file
